/* @HEADER@ */
// ************************************************************************
// 
//                             Sundance
//                 Copyright 2011 Sandia Corporation
// 
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Kevin Long (kevin.long@ttu.edu)
// 

/* @HEADER@ */

#include "SundanceGaussianQuadratureType.hpp"
#include "SundanceGaussianQuadrature.hpp"
#include "SundanceGauss1D.hpp"
#include "SundanceTriangleQuadrature.hpp"
#include "SundanceQuadQuadrature.hpp"
#include "SundanceTetQuadrature.hpp"
#include "SundanceBrickQuadrature.hpp"

using namespace Sundance;
using namespace Sundance;
using namespace Sundance;
using namespace Sundance;
using namespace Sundance;
using namespace Teuchos;

GaussianQuadratureType::GaussianQuadratureType()
  : QuadratureTypeBase()
{
  
}

XMLObject GaussianQuadratureType::toXML() const 
{
  XMLObject rtn("GaussianQuadratureType");
  return rtn;
}


bool GaussianQuadratureType::supportsCellType(const CellType& cellType) const
{
  switch(cellType)
    {
    case PointCell:
    case LineCell:
    case TriangleCell:
    case QuadCell:
    case TetCell:
    case BrickCell:
      return true;
    default:
      return false;
    }
}

int GaussianQuadratureType::maxOrder(const CellType& cellType) const
{
  switch(cellType)
    {
    case TetCell:
      return TetQuadrature::maxOrder();
    default:
      return -1;
    }
}


bool GaussianQuadratureType::hasLimitedOrder(const CellType& cellType) const
{
  switch(cellType)
    {
    case TetCell:
      return true;
    default:
      return false;
    }
}

bool GaussianQuadratureType::supports(const CellType& cellType, int order) const
{
  if (order <= 0) return false;

  switch(cellType)
    {
    case PointCell:
      return true;
    case LineCell:
      return true;
    case TriangleCell:
      return true;
    case QuadCell:
      return true;
    case TetCell:
      return TetQuadrature::supportsOrder(order);
    case BrickCell:
       return true;
    default:
      return false;
    }
}

QuadratureFamily GaussianQuadratureType::createQuadFamily(int order) const
{
  return new GaussianQuadrature(order);
}
