// Copyright (c) 2013, Sandia Corporation.
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// 
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
// 
//     * Redistributions in binary form must reproduce the above
//       copyright notice, this list of conditions and the following
//       disclaimer in the documentation and/or other materials provided
//       with the distribution.
// 
//     * Neither the name of Sandia Corporation nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 

#include <stk_util/stk_config.h>
#include <stk_util/diag/WriterExt.hpp>
#include <stk_util/diag/String.hpp>     // for Identifier, String
#include <stk_util/diag/StringUtil.hpp> // for demangle
#include <stk_util/util/Writer.hpp>     // for operator<<, Writer
#include "stk_util/parallel/MPI.hpp"    // for Loc, TempLoc



namespace stk {
namespace diag {

Writer &
operator<<(
  Writer &                      dout,
  const std::type_info &        t)
{
  if (dout.shouldPrint())
    dout << sierra::demangle(t.name());
  return dout;
}


Writer &
operator<<(
  Writer &                      dout,
  const sierra::String &        s)
{
  if (dout.shouldPrint()) 
    dout << s.c_str();
  return dout;
}


Writer &
operator<<(
  Writer &                      dout,
  const sierra::Identifier &    s)
{
  if (dout.shouldPrint())
    dout << s.c_str();
  return dout;
}


#if defined ( STK_HAS_MPI )
Writer &
operator<<(
  Writer &        dout,
  const sierra::MPI::Loc<int> &      loc) 
{
  if (dout.shouldPrint()) 
    dout << loc.m_value << "@" << loc.m_loc;
  return dout;
}


Writer &
operator<<(
  Writer &        dout,
  const sierra::MPI::Loc<double> &   loc)
{
  if (dout.shouldPrint())
    dout << loc.m_value << "@" << loc.m_loc;
  return dout;
}


Writer &
operator<<(
  Writer &        dout,
  const sierra::MPI::Loc<float> &    loc)
{
  if (dout.shouldPrint())
    dout << loc.m_value << "@" << loc.m_loc;
  return dout;
}

  
Writer &
operator<<(
  Writer &        dout,
  const sierra::MPI::TempLoc &   loc)
{
  if (dout.shouldPrint())
    dout << loc.m_value << " " << loc.m_other << "@" << loc.m_loc;
  return dout;
}
#endif

} // namespace diag
} // namespace stk
